/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This example consists of 2 programs. One (below) calls ossPrintPER()
 * and the other uses a PDU value notation in ranap.asn to quickly test
 * it.
 */

#include "ranap.h"

#define PRINT_PER
#ifdef PRINT_PER
#include "printasn.h"
#endif /* PRINT_PER */

//#define xxxx
#ifndef xxxx
#define FROM_FILEx
#define PDU RANAP_PDU_PDU
#define RULES OSS_PER_ALIGNED

extern long read_file(unsigned char *ib, FILE *filep);

unsigned char pdu[] = {
0x00, 0x00, 0x40, 0x4F, 0x00, 0x00, 0x01, 0x00, 0x36, 0x40, 0x48, 0x00,
0x00, 0x01, 0x00, 0x35, 0x00, 0x3E, 0x00, 0x2D, 0x80, 0x20, 0x2F, 0xA7,
0x20, 0x2F, 0xA8, 0x00, 0x00, 0xF4, 0x4C, 0x64, 0x06, 0x04, 0x80, 0x00,
0x51, 0x40, 0x00, 0x27, 0x40, 0x00, 0x00, 0x20, 0x28, 0x24, 0x00, 0x67,
0x40, 0x00, 0x00, 0x40, 0x00, 0x00, 0x22, 0x28, 0x24, 0x00, 0x3C, 0x40,
0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x64, 0x3C, 0x03, 0xFF, 0xFE, 0x1A,
0x00, 0x09, 0x31, 0x28, 0x04, 0x00, 0x1C, 0x00, 0x40, 0x01, 0x00

    
/* 0x00, 0x09, 0x40, 0x0E, 0x00, 0x00, 0x02, 0x00, 0x04, 0x40, 0x02, 0x01,
0x40, 0x00, 0x03, 0x40, 0x01, 0x80 */
};



int main(int argc, char **argv)
{

    OssGlobal  w,                /* OSS's global structure */
              *world = &w;       /* pointer to OSS's global structure */
    OssBuf    encodedData;       /* length and address of encoded data */
    void     *myCardPtr = NULL; /* address of decoded data */
    int	      pdu_num = PDU; /* PDU number */
    int	      retcode = 0;       /* return code */
    FILE     *fp = NULL;
    long      length;


#ifdef FROM_FILE
    char     *buf;               /* address of the encoder output buffer */

    if (argc != 2) {
	printf("Usage: gendec file-containing-encoded-data\n\n");
	return 1;
    }
#endif /* FROM_FILE */
	/*
	 * Call ossinit() first to initialize OSS's global structure
	 */
    if (retcode = ossinit(world, ranap)) {
	ossPrint(world, "Ossinit() returned %d\n", retcode);
	return retcode;
    }
    ossSetEncodingFlags(world, DEBUGPDU|AUTOMATIC_ENCDEC);
    ossSetDecodingFlags(world, DEBUGPDU|AUTOMATIC_ENCDEC);
    ossSetEncodingRules(world, RULES);

#ifdef FROM_FILE
    encodedData.value = buf = malloc(3000000);
    fp = fopen(argv[1], "rb");
    if (!fp) {
	ossPrint(world, "Can't open an encoded PDU file.\n");
	return 1;
    }
    encodedData.length = length = read_file(buf, fp);
    fclose(fp);
#else
    encodedData.value = pdu;
    encodedData.length = length = sizeof(pdu);
#endif /* FROM_FILE */
    ossPrintHex(world, (char *)encodedData.value, encodedData.length);

#ifdef PRINT_PER
    if (ossGetEncodingRules(world) == OSS_PER_ALIGNED ||
		ossGetEncodingRules(world) == OSS_PER_UNALIGNED) {

	ossPrint(world, "\nOutput from the PER print utility...\n\n");
	if (retcode = ossPrintPER(world, &pdu_num, &encodedData, NULL,
		    OSS_PRINT_OFFSET | OSS_HEXBYTES | 
		    OSS_PRINT_TYPE_INFO | OSS_PRINT_COMMENTS, NULL)) {
	    ossPrint(world, "ossPrintPER() failed with return code = %d\n%s", retcode, ossGetErrMsg(world));
	    return 1;
	}
    }
#endif /* PRINT_PER */

		/*
		 * Save the address of the buffer returned by the encoder
		 */
    ossPrint(world,"Length of encoded data = %d\n", encodedData.length);
    ossPrint(world,"\nThe decoder's trace messages...\n\n");
    if (ossDecode(world, &pdu_num, &encodedData, &myCardPtr)) {
	ossPrint(world, "%s\n", ossGetErrMsg(world));
#ifdef FROM_FILE
	free(buf);
#endif /* FROM_FILE */
//	memoryReport();
#if defined(_DLL)
	ossWterm(world);
#else
	ossterm(world);
#endif /* _DLL */
	return 1;
    }
    else {
		/*
		 * Print the decoder's output
		 */
	ossPrint(world, "\nOutput from the decoder...\n\n");
	ossPrintPDU(world, pdu_num, myCardPtr);

	ossPrint(world,"\nThe encoder's trace messages...\n\n");
	encodedData.length = 0;
	encodedData.value = NULL;
	ossEncode(world, pdu_num, myCardPtr, &encodedData);

	ossFreePDU(world, pdu_num, myCardPtr);
	ossPrint(world,"\nDecoding the PDU again...\n\n");
#ifdef FROM_FILE
	encodedData.value = buf;
#else
	encodedData.value = pdu;
#endif
	encodedData.length = length;
	myCardPtr = NULL;
	length = 0;
	if (decode(world, &pdu_num, (char **)&encodedData.value,
		&encodedData.length, &myCardPtr, &length, ranap,
							DEBUGPDU|AUTOMATIC_ENCDEC, NULL)) {
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
#ifdef FROM_FILE
	    free(buf);
#endif /* FROM_FILE */
#if defined(_DLL)
	    ossWterm(world);
#else
	    ossterm(world);
#endif /* _DLL */
	    return 1;
	}
	ossPrint(world, "\nOutput from the decoder...\n\n");
	ossPrintPDU(world, pdu_num, myCardPtr);
	freePDU(world, pdu_num, myCardPtr, ranap);
    }
		/*
		 * Free the encoder's output buffer
		 */
#ifdef FROM_FILE
    free(buf);
#endif /* FROM_FILE */
#if defined(_DLL)
    ossWterm(world);
#else
    ossterm(world);
#endif /* _DLL */
    return retcode;
}


long read_file(unsigned char *ib, FILE *filep)
{

    long len = 0;
    int c;


    for (; (c = fgetc(filep)) != EOF; ++len) {
      ib[len] = c;
    }
    return len;
}

#else

void *DLL_ENTRY mygetmem(size_t size);
void DLL_ENTRY myrlsmem(void *buf);
int main(int argc, char **argv)
{

    OssGlobal  w,                /* OSS's global structure */
              *world = &w;       /* pointer to OSS's global structure */
    int	      retcode = 0;       /* return code */


    if (retcode = ossinit(world, ranap)) {
	ossPrint(world, "Ossinit() returned %d\n", retcode);
	return retcode;
    }
    world->mallocp = mygetmem;
    world->freep = myrlsmem;
    ossSetFlags(world, NOCONSTRAIN);
    ossterm(world);
    exit(0);
}


/*
 * The following two functions may be defined to allocate/free memory
 * at low level.
 */
void *DLL_ENTRY_FDEF mygetmem(size_t size)
{

    return malloc(size);
}


void DLL_ENTRY_FDEF myrlsmem(void *buf)
{

    free(buf);
}
#endif /* xxxx */

